#!/usr/bin/perl

#    manor.pl - Manor utility
#    Copyright (C) 2000 Michael Jung
#    e-mail: miju@phantasia.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use CGI;
use CGI::Carp qw(fatalsToBrowser);
$| = 1;

$FILE_LOCATION = "../cgi-tmpl/manor/";

#########
### INPUT
#########

$q = new CGI;

foreach (1..207) {
    ${"A$_"} = $q->param("\$A$_") if ($q->param("\$A$_") ne "");
}
foreach (1..105) {
    ${"B$_"} = $q->param("\$B$_") if ($q->param("\$B$_") ne "");
    ${"B0$_"} = $q->param("\$B0$_") if ($q->param("\$B0$_") ne "");
}
foreach my $i (24, 38, 52, 66, 80, 94) {
    foreach my $j ("a".."e") {
        ${"A$i$j"} = $q->param("\$A$i$j") if ($q->param("\$A$i$j") ne "");
    }
}

#########
### CONSTANTS
#########

# NOTES: Glebe is always rolled
#  Assume 50:50 field:pasture
#  Everything not a bailiff is a vassal

# max members of household (+abcde)
$N = 14;

# max households
$M = 45;

# wood acres in topography
%woods = ( "Lowland" => 10,
	   "Highland" => 20,
	   "Coastal" => 10,
	   "Forest" => 50 );

# household support costs
%supp = ( "Fiefholder" => 3000,
	  "Spouse" => 2000,
	  "Lady-in-waiting" => 1500,
	  "Offspring" => 1000,
	  "Squire" => 1200,
	  "Bailiff" => 2000,
	  "Steward" => 2000,
	  "Herald" => 1200 +d(2)*300,
	  "Chamberlain" => 800,
	  "Alewife" => 500,
	  "Cook" => 400,
	  "Domestic" => 300,
	  "Baker" => 400,
	  "Clothier" => 900,
	  "Gardener" => 400,
	  "Ostler" => 1300,
	  "Nursemaid" => 400,
	  "Priest" => 800 + d(2)*700,
	  "Harper" => 600,
	  "Hideworker" => 900,
	  "Litigant" => 1100,
	  "Mason" => 1500,
	  "Metalsmith" => 1100,
	  "Physician" => 1200,
	  "Sage/Tutor" => 1200,
	  "Scribe" => 800,
	  "Salter" => 700,
	  "Teamster" => 900,
	  "Timberwright" => 1200,
	  "Weaponcrafter" => 1800,
	  "Woodcrafter" => 1000,
	  "Warhorse" => 1800,
	  "Palfrey" => 900,
	  "Lord's Kennel" => 600,
	  "Lord's Falconry" => 800,
	  "Light Foot" => 576,
	  "Medium Foot" => 936,
	  "Shortbow" => 936,
	  "Longbow" => 1296,
	  "Light Horse" => 2016,
	  "Medium Horse" => 3200,
	  "Heavy Horse" => 4608 );

# relatives, nobles, servants, craftsmen
@suppr = ("Fiefholder", "Spouse", "Lady-in-waiting", "Offspring", "Relative");
@suppn = ("Squire", "Bailiff", "Steward" );
@suppn2 = (@suppr,@suppn);
@supps = ("Chamberlain", "Cook", "Alewife", "Domestic", "Baker", "Gardener",
	  "Nursemaid");
@suppc = ("Clothier", "Ostler", "Priest", "Harper", "Herald", "Hideworker",
	  "Litigant", "Mason", "Metalsmith", "Physician", "Sage/Tutor",
	  "Scribe", "Salter", "Teamster", "Timberwright", "Weaponcrafter",
	  "Woodcrafter");

# occupation
%occup = ( "Craftsman" => [0, sub{0}, sub{5*d(6)}, 10],
	   "Farmer" => [0, sub{0}, sub{int(1+d(5)/5)*5*(d(6)+d(6))}, 25],
	   "Villein" => [1, sub{d(20)+10}, sub{(d(5)==1)?(5*d(6)):0}, 60],
	   "Half-Villein" => [0, sub{d(10)+10}, sub{0}, 80],
	   "Cottar" => [-1, sub{d(6)-1}, sub{0}, 90],
	   "Slave" => [0, sub{0}, sub{0}, 100],
	   "Thrall" => [0, sub{0}, sub{0}, 100],
	   "Yeoman (LF)" => [0, sub{0}, sub{10+d(10)}, 101],
	   "Yeoman (MF)" => [0, sub{0}, sub{20+d(10)}, 101],
	   "Yeoman (LB)" => [0, sub{0}, sub{40+d(10)}, 101],
	   "Yeoman (LH)" => [0, sub{0}, sub{60+d(10)}, 101] );

# Craftsmen [roll,fee]
%craft = ( "Miller" => [25,240],
	   "Metalsmith" => [45,144],
	   "Woodcrafter" => [60,120],
	   "Salter" => [70,120],
	   "Hideworker" => [75,144],
	   "Timberwright" => [80,216],
	   "Charcoaler" => [85,180],
	   "(Craftsman, GM)" => [90,0],
	   "Innkeeper" => [95,216],
	   "Shipwright" => [100,144] );

# Crops [roll,yield,labor]
@crop = ( [10, 45, 5],
	  [25, 48, 6],
	  [40, 42, 5],
	  [65, 40, 5],
	  [75, 80, 8],
	  [80, 66, 6],
	  [85, 72, 6],
	  [100, 90, 7] );

# Livestock [roll,yield,head,labor]
@live = ( [0, 130, .5, 17],
	  [10, 188, .5, 20],
	  [40, 40, 2, 5],
	  [80, 33, 3, 3],
	  [100,15, 10, 2] );

# Feudal obligations
%feudal = ("Kanday" => 600,
	   "Rethem" => 600,
	   "Melderyn" => 600,
	   "Kaldor" => 600,
	   "Chybisa" => 300,
	   "Tharda" => 600,
	   "Azadmere" => 300,
	   "Ivinia" => 300,
	   "Orbaal" => 300,
	   "Harbaal" => 300,
	   "Chelemby" => 100 );
# yeoman
%yeoman = ( "Yeoman (LF)" => [50,2,sub{10+d(10)}],
	    "Yeoman (MF)" => [70,3,sub{20+d(10)}],
	    "Yeoman (LB)" => [90,6,sub{40+d(10)}],
	    "Yeoman (LH)" => [100,6,sub{60+d(10)}] );

#########
### CALCULATIONS
#########

# Holder
$A202 ="Sir Default" if (precond("!A202"));

# Gross acres (min 1200 max 2399)
$A1 = 1500 if (precond("!A1"));

# Topography
$A2 = "Lowland" if (precond("!A2"));

# LQ
$A9 = 1.00 if (precond("!A9"));

# FI
$A10 = 1.00 if (precond("!A10"));

# TI
$A207 = 0.50 if (precond("!A207"));

# Realm
$A200 = "Default" if (precond("!A200"));

# Year
$A205 = 720 if (precond("!A205"));

# Fief
$A203 = "Default" if (precond("!A203"));

# Feudal income
$A125 = 0 if (precond("!A125"));

# Wood acres
$A3 = round( ($woods{$A2} - 5 + d(10)) * $A1/100) if (precond("!A3"));

# Assart (2% of wood acres)
$A122 = round( 0.02 * $A3) if (precond("!A122"));

# Labor spent on woods
$A95 = $A3 * 3 if (precond("!A95"));

# Cleared Acres
$A4  = $A1 - $A3 if (precond("!A4"));

# Crop acres planned coming year
$A119 = round( $A4/2 ) if (precond("!A119"));

# Pasture acres planned coming year
$A120 = round( $A4/2 ) if (precond("!A120"));

# Labor spent on crops
$A97 = round( .5 * $A4 * 6 ) if (precond("!A97"));

# Labor spent on pasture
$A98 = round( .5 * $A4 * 10 ) if (precond("!A98"));

# Number of HD
$A204 = round( $A4/40 * $A9 ) if (precond("!A204"));

# Lord household

foreach my $j (11..24, "24a", "24b", "24c", "24d", "24e") {
    if (precond("A".$j)) { push(@t, $ {"A".$j}); }
    else { push (@ct, "A".$j); }
}

$ep = 3 * int(($A1 - 1200) / 300);
while ($#ct > 0) {
    my $i = shift @ct;
# Basic persons
    my $t;
    if ($#suppr > 0) { $t = shift @suppr; }
    elsif ($#supps > 2) { $t = shift @supps; }
# Extended persons
    elsif ($ep > 0) {
	$ep--;
	if (($#suppn > 0) && ($#ct%3 == 0)) { $t = shift @suppn; }
        elsif (($#suppc > 0) && ($#ct%3 == 1)) { $t = shift @suppc; }
        elsif (($#supps > 0) && ($#ct%3 == 2)) { $t = shift @supps; }
    }
    if (grep(/$t/, @t) == 0) {
	$ {"$i"} = $t;
        push(@t, $t);
    }
    elsif ($t ne "") {
        unshift @ct, $i;
    }
}

$A24d = "Warhorse" if (grep(/Warhorse/, @t) == 0 && precond("!A24d"));
$A24e = "Palfrey" if (grep(/Palfrey/, @t) == 0 && precond("!A24e"));

# Lord household (derivatives)
my $pn = 0;

for my $j (1..$N) {
    if (precond("A".(10+$j))) {
# ML
	$ {"A".(24+$j)} = 25 + 5*(d(6) + d(6) + d(6))
	    if (precond("!A".(24+$j)));
        $ {"A".(24+$j)} = "-" if ( ($ {"A".(10+$j)} =~ /orse/) > 0 ||
                                   ($ {"A".(10+$j)} eq "Palfrey") );
# Each
	$ {"A".(66+$j)} = $supp{ $ {"A".(10+$j)}}
	    if (precond("!A".(66+$j)));
# #
        if (precond("!A".(38+$j))) {
	    $ {"A".(38+$j)} = ($ {"A".(10+$j)} eq "Domestic") ? 5 : 1;
            $rpn = "A".(38+$j) if ($ {"A".(10+$j)} eq "Palfrey");
            $ {"A".(38+$j)} = d(3) if ($ {"A".(10+$j)} eq "Offspring");
        }
# Total
	$ {"A".(80+$j)} = $ {"A".(66+$j)} * $ {"A".(38+$j)}
	    if (precond("!A".(80+$j)." && A".(66+$j)." && A".(38+$j)));
# Loyal
	$ {"A".(52+$j)} = 50 + d(6) + d(6) + d(6) + d(6) + d(6)
	    + ((grep $ {"A".(10+$j)},@suppr) ? 0 : 100)
		if (precond("!A".(52+$j)));
	$ {"A".(52+$j)} = "-" if ( ($ {"A".(10+$j)} eq "Fiefholder") ||
                                   (($ {"A".(10+$j)} =~ /orse/) > 0) ||
                                   ($ {"A".(10+$j)} eq "Palfrey") );
# Total
	$A206 += $ {"A".(80+$j)} if (precond("A".(80+$j)));
    }
    else {
	$ {"A".(10 + $j)} = " ";
	$ {"A".(24 + $j)} = " ";
	$ {"A".(66 + $j)} = " ";
	$ {"A".(38 + $j)} = " ";
	$ {"A".(80 + $j)} = " ";
	$ {"A".(52 + $j)} = " ";
    }
}

for my $j ("a".."e") {
    if (precond("A24".$j)) {
# ML
	$ {"A38".$j} = 25 + 5*(d(6) + d(6) + d(6))
	    if (precond("!A38".$j));
        $ {"A38".$j} = "-" if ( ($ {"A24".$j} =~ /orse/) > 0 ||
                                ($ {"A24".$j} eq "Palfrey") );
# Each
	$ {"A80".$j} = $supp{ $ {"A24".$j}}
	    if (precond("!A80".$j));
# #
        if (precond("!A52".$j)) {
	    $ {"A52".$j} = ($ {"A24".$j} eq "Domestic") ? 5 : 1;
            $rpn = "A52".$j if ($ {"A24".$j} eq "Palfrey");
            $ {"A52".$j} = d(3) if ($ {"A24".$j} eq "Offspring");
        }
# Total
	$ {"A94".$j} = $ {"A80".$j} * $ {"A52".$j}
	    if (precond("!A94".$j." && A80".$j." && A52".$j));
# Loyal
	$ {"A66".$j} = 50 + d(6) + d(6) + d(6) + d(6) + d(6)
	    + ((grep $ {"A24".$j},@suppr) ? 0 : 100)
		if (precond("!A66".$j));
	$ {"A66".$j} = "-" if ( ($ {"A24".$j} eq "Fiefholder") ||
                                (($ {"A24".$j} =~ /orse/) > 0) ||
                                ($ {"A24".$j} eq "Palfrey") );
# Total
	$A206 += $ {"A94".$j} if (precond("A94".$j));
    }
    else {
	$ {"A24".$j} = " ";
	$ {"A38".$j} = " ";
	$ {"A80".$j} = " ";
	$ {"A52".$j} = " ";
	$ {"A94".$j} = " ";
	$ {"A66".$j} = " ";
    }
}

# determine number of palfreys
foreach my $j (1..14) {
    if (precond("A".(10 + $j))) {
	$pn += (grep(/${"A".(10 + $j)}/, @suppn2) > 0) * $ {"A".($j + 38)};
    }
}

${"$rpn"} = $pn;

# Yeomen (prepare)
$i = 600 if (!($i = $feudal{$A200}));
$YEO = 2 * int ($A1 / $i);

# Tenant households (max 45)
for my $i (1..(($A204 > $M) ? $M : $A204)) { 
    $ {"B0".$i} = $i;
# Occupation
    my $r = d(100);
    if (precond("!B1".$i)) {
	foreach my $k (sort { $ {$occup{$b}}[3] <=> $ {$occup{$a}}[3] }
		       keys %occup) {
	    if ($r <= $ {$occup{$k}}[3]) {
		$ {"B1".$i} = $k;
	    }
	}
# (Farmer / Yeoman details)
	if (($ {"B1".$i} eq "Farmer") && ($YEO > 0)) {
	    my $j = 0;
	    my $r = d(100);
	    foreach my $k (sort { $ {$yeoman{$b}}[0] <=> $ {$yeoman{$a}}[0] }
			   keys %yeoman) {
		if ($r <= $ {$yeoman{$k}}[0]) {
		    $ {"B1".$i} = $k;
		    $j = $ {$yeoman{$k}}[1];
		}
	    }
	    $YEO -= $j;
	    $RPA = 1;
	    $ {"B9".$i} = "military service 60-90 days";
	}
	else {
	    $RPA = 6;
	}
# (Slave details)
	$ {"B1".$i} = "Cottar"
	    if (($ {"B1".$i} eq "Slave") && ($A200 ne "Tharda")
		&& ($A200 ne "Rethem") && ($A200 ne "Orbaal"));
	if ($ {"B1".$i} eq "Slave") {
	    if (precond("!B6".$i)) {
		my $M = ($ {"B3".$i} - 2);
		$ {"B6".$i} = 600 - 60 * round(2*$M/3);
		$ {"B6".$i} += int (150 * $M);
	    }
	    $ {"B9".$i} = (240 + round(280*$M/3))."d cost"
		if (precond("!B9".$i));
	}
	$ {"B1".$i} = "Thrall"
	    if (($A200 eq "Orbaal") && ($ {"B1".$i} eq "Slave"));
    }
    $ {"B9".$i} = " " if (precond("!B9".$i));
# ML
    $ {"B2".$i} = 25 + 5*(d(6) + d(6) + d(6)) if (precond("!B2".$i));
# Size
    if (precond("!B3".$i)) {
	$ {"B3".$i} = d(6) + 1 + $ {$occup{$ {"B1".$i}}}[0];
	$ {"B3".$i} += d(6) - 1 if ($ {"B3".$i} eq 7);
    }
# Acres
    $ {"B4".$i} = &{$occup{$ {"B1".$i}}[1]} if (precond("!B4".$i));
    $ {"B5".$i} = &{$occup{$ {"B1".$i}}[2]} if (precond("!B5".$i));
# Labor days
    $ {"B6".$i} = 4 * $ {"B4".$i} if (precond("!B6".$i));
# Rent
    $ {"B7".$i} = 60 + $RPA * $ {"B5".$i} if (precond("!B7".$i));
# Fees
    my $new = 0;
    $new = 1, $ {"B8".$i} = 6 + $ {"B4".$i} + $ {"B5".$i}
        if (precond("!B8".$i));
# Craftsmen details
    if ($ {"B1".$i} eq "Craftsman") {
	my $r = d(100);
	my $rs = ($A2 eq "Coastal") ? 1.00 : 0.95;
	foreach my $j (sort { $ {$craft{$b}}[0] <=> $ {$craft{$a}}[0] }
		       keys %craft) {
	    $ {"B1".$i} = $j, $fee = $ {$craft{$j}}[1]
		if ( $r <= ($ {$craft{$j}}[0] / $rs) );
	}
	$ {"B8".$i} += $fee if ($new);
	$ {"B9".$i} = "fees include license"
	    if (index($ {"B1".$i}, "(") == -1);
    }
}

# Glebe (cont.)
$i = 95 if ( ($i = 5 * $A204) > 95 );

if (d(100) <= $i) {
    my $i = d($A204);
    $ {"B1".$i} = "Priest";
    $ {"B4".$i} = $ {"B6".$i} = $ {"B7".$i} = $ {"B8".$i} = 0;
    $ {"B5".$i} = 5*(2 + int rand(6) + int rand(6));
    $PRIEST = $i;
}

# free slots
for my $i ($A204+1..$M) { 
    $ {"B0".$i} = " " if (precond("!B0".$i));
    $ {"B1".$i} = " " if (precond("!B1".$i));
    $ {"B2".$i} = " " if (precond("!B2".$i));
    $ {"B3".$i} = " " if (precond("!B3".$i));
    $ {"B4".$i} = " " if (precond("!B4".$i));
    $ {"B5".$i} = " " if (precond("!B5".$i));
    $ {"B6".$i} = " " if (precond("!B6".$i));
    $ {"B7".$i} = " " if (precond("!B7".$i));
    $ {"B8".$i} = " " if (precond("!B8".$i));
    $ {"B9".$i} = " " if (precond("!B9".$i));
}

# Total
foreach (1..$M) {
    $B100 += $ {"B3".$_};
    $B101 += $ {"B4".$_};
    $B102 += $ {"B5".$_};
    $B103 += $ {"B6".$_};
    $B104 += $ {"B7".$_};
    $B105 += $ {"B8".$_};
}

# Glebe
$ {"B9".$PRIEST} = (60 + 5 * ($B101 + $B102))."d Glebe"
    if (defined($PRIEST));

# Tenant acres
$A5 = $B101 + $B102 if (precond("!A5"));

# Demesne Acres
$A6 = $A4 - $A5 if (precond("!A6"));

# Labor Pool
$A7 = $A204 * 500 if (precond("!A7"));

# Labor Obligation
$A8 = $B103 if (precond("!A8"));

# WI
if (precond("!A201")) {
    $A201 -= .05 if ( ($A201 = .65 + (d(6) + d(6) + d(6)) * .05) > 1 );
}

# Woods / acres
if (precond("!A115")) {
    $A115 = $A3 if ( ($A115 = round($A95 / 3)) > $A3 );
}

# Woods / yield
$A134 = round(18 * $A9 * $A10 * $A201) if (precond("!A134"));

# Woods / kind
$A101 = $A134 * $A115 if (precond("!A101"));

# Crops / acres
if (precond("!A116")) {
    $A116 = round($A4 / 2) if ( ($A116 = round($A97 / 6)) > round($A4 / 2) );
}

# Crops / yield
$A135 = round(60 * $A9 * $A10 * $A201) if (precond("!A135"));

# Crops / kind
$A100 = $A135 * $A116 if (precond("!A100"));

# Pasture / acres
if (precond("!A117")) {
    $A117 = $A4 - $A116 if ( ($A117 = round($A98 / 10)) > $A4 - $A116);
}

# Pasture / yield
$A136 = round(90 * $A9 * $A10 * $A201) if (precond("!A136"));

# Pasture / kind
$A99  = $A136 * $A117 if (precond("!A99"));

# Waste
$A118 = $A4 - $A116 - $A117; # - $A115

# Crop acres (prepare)
$i = 0;
foreach my $j (176..183) {
    $ {"A".$j} = 0 if (precond("!A".$j));
    $i += abs($ {"A".$j});
}

# Crop acres (cont.)
while ($i < ($A116 - 4) ) { # 4 = .5*8 for rounding effects
    my $r = d(100);
    my $k = 0; $k++ while ($r > $crop[$k]->[0]);
    $ {"A".(183-$k)} += round($A116 / 10);
    $i += abs($A116 / 10);
}

foreach my $j (0..7) {
# Crop yield
    $ {"A".(199-$j)} = round($crop[$j]->[1] * $A9 * $A10 * $A201)
	if (precond("!A".(199-$j)));
# Crop labor (& total)
    if (precond("!A".(184+$j))) {
	$ {"A".(184+$j)} = round($ {"A".(183-$j)} * $crop[$j]->[2]);
	$A174 += $ {"A".(184+$j)};
    }
# Crop kind (& total)
    if (precond("!A".(172-$j))) {
	$ {"A".(172-$j)} = round($ {"A".(183-$j)} * $ {"A".(199-$j)});
	$A173 += $ {"A".(172-$j)};
    }
# Crop acres
    $A175 += $ {"A".(183-$j)};
}

# Livestock acres (prepare)
$i = 0;

$A154 = round($A117 / 5) if (precond("!A154")); # oxen
foreach my $j (150..154) {
    $ {"A".$j} = 0 if (precond("!A".$j));
    $i += abs($ {"A".$j});
}

# Livestock acres (cont.)
while ($i < ($A117 - 2.5) ) { # 2.5 = .5*5 for rounding effects
    my $r = d(100);
    my $k = 0; $k++ while ($r > $live[$k]->[0]);
    $ {"A".(154-$k)} += round($A117 / 10);
    $i += abs($A117 / 10);
}
foreach my $j (0..4) {
# Livestock yield
    $ {"A".(155+$j)} = round($live[$j]->[1] * $A9 * $A10 * $A201)
	if (precond("!A".(155+$j)));
# Livestock head
    $ {"A".(160+$j)} = round($live[$j]->[2] * $ {"A".(154-$j)})
	if (precond("!A".(160+$j)));
# Livestock labor
    if (precond("!A".(143+$j))) {
	$ {"A".(143+$j)} =  $ {"A".(160+$j)} * $live[$j]->[3];
	$A149 += $ {"A".(143+$j)};
    }
# Livestock kind
    if (precond("!A".(137+$j))) {
	$ {"A".(137+$j)} =  $ {"A".(160+$j)} * $ {"A".(155+$j)};
	$A142 += $ {"A".(137+$j)};
    }
    $A148 += $ {"A".(154-$j)};
}

# Total harvest (labor / kind)
$A108 = $A95 + $A97 + $A98 if (precond("!A108"));
$A102 = $A101 + $A99 + $A100 if (precond("!A102"));

# Crop seed
$A103 = $A119 * 12 if (precond("!A103"));

# Winter feed
$A104 = $A120 * 12 if (precond("!A104"));

# Fief maintenance (acres / labor / kind)
$A121 = $A4 if (precond("!A121"));
$A109 = $A121 * 3 * $A10 if (precond("!A109"));
$A105 = $A121 * 6 * $A10 if (precond("!A105"));

# Assart (labor / kind)
$A110 = $A122 * 30 if (precond("!A110"));
$A106 = $A122 * 120 if (precond("!A106"));

# Fief income (labor / kind)
$A111 = $A108 + $A109 + $A110 if (precond("!A111"));
$A111 .= "(!)" if ($A111 > $A7);
$A107 = $A106 + $A102 - $A103 - $A104 - $A105 if (precond("!A107"));

# Demesne income
$A112 = round($A107 * $A6 / $A4) if (precond("!A112"));

# Tenant rent & fees
$A113 = $B104 + $B105 if (precond("!A113"));

# Glebe revenue
$A114 = (defined($PRIEST)) ? round($ {"B9".($PRIEST)}/2) : 0
    if (precond("!A114"));

# Taxes & tolls
$A123 = (d(6) + d(6) + d(6)) * 240 * $A207 if (precond("!A123"));

# Amercement
$A124 = $A204 * (d(6) + d(6)) * 5 if (precond("!A124"));

# Total income
$A126 = $A112 + $A113 + $A114 + $A124 if (precond("!A126"));

# Household (cost)
$A127 = $A206 if (precond("!A127"));

# Feudal payments
$A128 = ($A203 =~ /Bailiff/) > 0 ? round( $A126 / 3 ) : (3 * $A1)
    if (precond("!A128"));

# Tithe
$A129 = round( $A126 / 10 ) if (precond("!A129"));

# Labor hired
if (precond("!A133")) {
    $A133 = 0 if ( ($A133 = $A6 * 8 - $A8) < 0 );
}

# Political expenses
$A130 = $A1 if (precond("!A130"));

# Total expenses
$A131 = $A127 + $A128 + $A129 + $A133 + $A134 if (precond("!A131"));

# Privy purse
$A132 = $A126 - $A131 if (precond("!A132"));

sub d { return 1 + int rand @_[0]; }

sub precond { $_ = shift; s/([A-Za-z0-9]+)/"defined(\$$1)"/eg; return eval; }

sub round { return sprintf "%d", @_[0] + .5; }

#########
### OUTPUT
#########

$gs = "| gs -q -dNOPAUSE -dBATCH -sDEVICE=\$d -sOutputFile=- -c save pop -";

# PREPARE (STD)OUT

%action = ("HTML" => [".html","text/html"],
	   "PS" => [".ps","application/postscript"],
	   "FIG" => [".fig","application/fig"],
	   "JPEG" => [".ps","image/jpeg"],
	   "PDF" => [".ps","application/pdf"],
	   "FORM" => ["f.html","text/html"]);

open(FILE, "<".$FILE_LOCATION."manor".$action{$q->param("action")}[0]) ||
    die "Can't open file: $!";
print $q->header(-type => $action{$q->param("action")}[1],
		 -attachment => "manor".$action{$q->param("action")}[0]);

if ($q->param("action") eq "HTML") {
    while (my $i = <FILE>) {
	$i =~ s/\$([A-B][0-9]+[a-e]?)/$ {"$1"}/;
	print $i;
    }
}
if ($q->param("action") eq "PS") {
    while (my $i = <FILE>) {
	$i =~ s/\$([A-B][0-9]+[a-e]?)/$ {"$1"}/;
	print $i;
    }
}
if ($q->param("action") eq "FIG") {
    while (my $i = <FILE>) {
	$i =~ s/\$([A-B][0-9]+[a-e]?)/$ {"$1"}/;
	print $i;
    }
}
if ($q->param("action") eq "JPEG") {
    $d = "jpeggray -r141"; $gs =~ s/\$d/$d/;
    open PDF, $gs;
    while (my $i = <FILE>) {
	$i =~ s/\$([A-B][0-9]+[a-e]?)/$ {"$1"}/;
	print PDF $i;
    }
    close(PDF);
}
if ($q->param("action") eq "PDF") {
    $d = "pdfwrite"; $gs =~ s/\$d/$d/;
    open PDF, $gs;
    while (my $i = <FILE>) {
	$i =~ s/\$([A-B][0-9]+[a-e]?)/$ {"$1"}/;
	print PDF $i;
    }
    close(PDF);
}
if ($q->param("action") eq "FORM") {
    while (my $i = <FILE>) {
	$i =~ s/\$([A-B][0-9]+[a-e]?)\" value=\"/\$$1\" value=\"$ {"$1"}/;
	print $i;
    }
}
close(FILE);
